gradient descent library
========================

## Description

This library has a simple C++ implementation of the simple [gradient descent algorithm](https://en.wikipedia.org/wiki/Gradient_descent).

The main features of this library are:

* Provides a function pointer to evaluate the desired function at a given point which can be set by the user (*set_function()*).
* Provides a function pointer to evaluate the derivative of the desired function at a given point which can be set by the user (*set_function_der()*).
* The user can set the coordinate range (maximum and minimum) of convergence of the algorithm as well as the maximum coordinate increment at each iteration (*set_coord_constraints()* and *set_max_coord_inc()*).

The function to compute the gradient algorithm is called *compute* and has the following prototype:

```
bool compute(double max_func_error,unsigned int max_iter,double start_point,bool &beyond_limits,bool max);
```

where:
* **max_func_error**: is the maximum difference with the previous iteration to consider that the algorithm has found a solution.
* **max_iter**: is the maximum number of iterations allowed to the algorihtm. If the solution is found before reacing this limit, the function will return immediatelly.
* **start_point**: is the start point of the algorithm. It may be usefull to avoid local minima of the desired function.
* **beyond_limits**: an output flag that indicates that the solution found is one of the convergence limits set by the user.
* **max**: an input flag to indicate whetehr to find the maximum (true) of the minimum (false) of the function. If not set, the default value is true.

The convergence algorithm can end in several different ways:

* The algorithm can not find a solution with the given maximum error and number of iterations. In this case the *compute()* function returns false;
* The algorithm finds a valid solution but it is one of the convergence limits set by the user. In this case the *compute()* function returns true, but the *beyond_limits* flag is set.
* The algorithm finds a valid solution inside the convergence limits. In this case the *compute()* function return true.

## How to use it
To us this class:

* Create an object of this class.
* Set the maximum coordinate increment at each iteration with the *set_max_coord_inc()* function. Is not set, the default value is 0.1.
* Set the desired coordinate range with the *set_coord_constraints()* function. If not set, the default values are max=double_max and min=double_min.
* Set the function evaluation function with the *set_function()* function. If not set, the algorithm will not work.
* Set the function derivative evaluation function with the *set_function_der()* function. If not set, the algorithm will not work.
* Execute the algorithm with the *compute()* function.

An example is provided for a simple sinus function.

