COpendriveLane
==============

## Description

This class describes a single lane of a road segment and its connectivity. In general, each lane is connected to a set of previous and next lanes, and for road segments with multiple lanes on each side, they are also connected to the adjacent lanes (lanes at opposite sides of a road segment are not considered adjacent).

The next two images show (as transparent green boxes) the set of COpendriveLane's defined for the example road presented in [here]().

<img src="images/lanes.png" alt="COpendriveLane objects created for the example road" width="647" height="547">

<img src="images/lanes_junction.png" alt="COpendriveLane objects created for a junction of the example road" width="622" height="457">

Each lane is assigned a unique index when it is created which depends only on the order in which they are imported from the Opendrive file (black number in the middle of the green boxes in the previous image). This index may only change when a new sub-road is generated, in which case the lanes in the new road are re-indexed. The lanes are also identified in the same way that Opendrive identifies them: negative integers values are used to identify right lanes, and positive values to identify left lanes. These values increase (for left lanes) or decrease (for right lanes) the further they are from the center lane of the segment (value in parenthesis in the previous images).

The second image shows a detail of the COpendriveLane objects created for the lower junction (junction0). The upper junction (junction1) is equivalent.

The lanes are defined in the direction of the traffic. That is, right lanes have the same heading as the road segment, but left lanes have opposite heading, starting at the end of the road segment and ending at the start.

The lanes can only have a constant width. Multiple width records on the opendirve road description file are not supported. Also, one speed record is required for each lane. If no speed is defined a default speed of 60Km/h will be used. Check the [main opendrive documentation](opendirve) for more information on the supported features.

## Main features

The main features of this class are:

* Provides access to all the road nodes of the lane defined as [COpendriveRoadNode](opendrive_road_node.md). The road nodes can be accessed by two different ways:
    * **by local index**: which is the local index of the road node inside the lane, not the unique index assigned to the road nodes when they are created.
    * **by the closest pose**: in this case when the position of interest is known (i.e. the position of the car, an obstacle, etc...), it is possible to get the road node which is closest to this position. It is possible to get the index of the road node of interest or a reference to it.
* Provides connectivity information of each lane with its neighbours (previous, next, left and right lanes). The Previous and next connecting lanes can only be accessed by their local index, not the one assigned to the lanes when they are created.
* Provides road mark information for the left and right sides, even if the left or right lanes don't exist.
* Provides a reference to the parent road segment [COpendriveRoadSegment](opendrive_road_segment.md).
* It is possible to find the closest position to the lane independently of the road element where it belongs.
* It is possible to get the pose and curvature at any point on the lane trajectory.
* In case of unexpected errors, this class throws an instance of CException. All other errors are reported to the user.

For further details on the implementation of these features check the doxygen API documentation.

## Closest nodes, poses and lane distances

At this point it is important to pay some attention to how the closest nodes, poses and lane distances are computed.

In this case, the origin of the reference system is placed at the center of the lane, at the beginning of the lane. For right lanes this means the origin is placed at the beginning of the road segment, and for left lanes it means that the origin is at the end of the road segment. The positive x axis (in red) is pointing toward the segment and the positive y axis (in green) is always pointing to the left (the positive z axis would be up). The next figure shows the reference system for left and right lanes for two road segment (a straight segment on the left and a curve segment on the right).

<img src="images/ref_system_lanes.png" alt="Closest poses and lane distances computation for lanes" width="1023" height="382">

The lane geometry is build from the lane link geometries of the links connecting the road nodes belonging to the lane. Therefore, the computation of the closest pose is equivalent to that of the links. See the [COpendrveLink](opendrive_link.md) documentation for more details.

In case the closest point exist, its **lane distance** is always computed from the origin of the reference system of the lane, following the path defined by the lane geometry (center of the lane) as shown in the previous image. Do not confuse this distance with the closest euclidean distance between the point of interest and the closest point belonging to the lane.

The closest road node to a given pose is the road node belonging to the lane whose lane link has the smallest euclidean distance to the given pose. Since the closest pose to a link is computed with respect to its starting road node, the closest road node will always be the one before the closest point, even though the distance to the next road node in the lane may be smaller. This case is shown in previous image, where the closest road nodes are shown in blue.

