COpendriveRoadSegment
=====================

## Description

This class describes a segment of the road in which the number of lanes at each side remains constant. In general, each road segment is connected to the adjacent road segments, but since the roads have to possible directions, the concept of next and previous is only defined when the side of the road is selected.

The next two images show (as transparent blue boxes) the set of COpendriveRoadSegments defined for the example road presented in [here]().

<img src="images/road_segments.png" alt="COpendriveRoadSegment objects created for the example road" width="606" height="549">

<img src="images/road_segments_junction.png" alt="COpendriveRoadSegment objects created for a junction of the example road" width="582" height="501">

Each road segment is assigned a unique index when it is created which depends only on the order in which they are imported from the Opendrive file (black number in the middle of the blue boxes in the previous image). A part from this index, each road can also be identified by the Opendrive identifier and name.

The second image shows a detail of the COpendriveRoadSegment objects created for lower junction (junction0). The upper junction (junction1) is equivalent. In this case the junction roads have a single right lane, so 6 road segments are necessary to fully connect the 3 roads.

The connectivity between all road segments is extracted from the following Opendrive information, so it is very important that this information is valid, otherwise, the process of loading the Opendrive description file may file or generate an undesired results.

* Road -> link -> predecessor
* Road -> link -> successor
* Junction -> connection
* Junction -> connection -> laneLink.

The last two points are only necessary for junction roads.

## Main features

The main features of this class are:

* The geometry of each road segment is described in two different ways.
    * as a set of [Opendrive geometry classes](), directly imported from the Opendrive file.
    * as a set of [G2 Splines](g2_splines.md), generated from the opendrive geometric information.
* Provides access to all the lanes of the road segment defined as [COpendriveLane](opendrive_lane.md) objects. The road segment lanes can be accessed by two different ways:
    * **by id**: in this case the lanes are identified in the same way that Opendrive identifies them: negative integers values are used to identify right lanes, and positive values to identify left lanes. These values increase (for left lanes) or decrease (for right lanes) the further they are from the center of the segment.
    * **by the closest pose**: in this case when the position of interest is known (i.e. the position of the car, an obstacle, etc...), it is possible to get the road segment lane which is closest to this position. It is possible to get the identifier of the road segment lane of interest or a reference to it.
* Provides access to all the road segment nodes of the road defined as [COpendriveRoadNode](opendrive_road_node.md) objects. The road nodes can be accessed by two different ways:
    * **by index**: which is the local index of the road node inside the road segment, not the unique index assigned to the road nodes when they are created.
    * **by the closest pose**: in this case when the position of interest is known (i.e. the position of the car, an obstacle, etc...), it is possible to get the road node which is closest to this position. It is possible to get the index of the road node of interest or a reference to it.
* It is possible to find the closest position to the road segment independently of the road element where it belongs.
* Provides connectivity information of each road segment with its neightbours. The connecting road segments can be accessed in two different ways:
    * **by index**: which is the local index of the connecting road segments inside the current road segment, not the unique index assigned to the road segments when they are created.
    * **previous** or **next**: in this case only the previous or next segment or segments are accessed, depending on the side of the road of interest.
* Provides access to all the traffic signs belonging to the road segment defined as [COpendriveSignals](opendrive_signals.md) objects. In this case it is only possible to access them by index.
* Provides access to all the obstacles belonging to the road segment defined as [COpendriveObjects](opendrive_objects.md) objects. In this case it is only possible to access them by index.
* It is possible to get the pose and curvature at any point on the lane trajectory.
* Provides a reference to the parent road [COpendriveRoad](opendrive_road.md) object.
* In case of unexpected errors, this class throws an instance of CException. All other errors are reported to the user.

For further details on the implementation of these features check the doxygen API documentation.

## Closest nodes, lanes, poses and segment distances

At this point it is important to pay some attention to how the closest nodes, lanes, poses and segment distances are computed.

In this case, the origin of the reference system is placed at the center lane (middle of the road segment) at one of its edges. The positive x axis (in red) is pointing toward the segment and the positive y axis (in green) is always pointing to the left (the positive z axis would be up). The next figure shows the reference system of two road segment (a straight segment on the left and a curve segment on the right).

<img src="images/ref_system_road_segments.png" alt="Closest poses and segment distances computation for road segments" width="985" height="437">

The left an right lanes are defined with respect to this reference frame for each road segment, so it is possible that left lanes of one segment connect to right lanes of another one, depending on the definition of their reference systems. This definitions coincides with the one used by Opendrive.

When finding the closest pose to a road segment, the desired position of interest (yellow points in the previous image) is projected (closest distance) to the geometry that describes the center lane of the segment (light red point in the previous image labeled S). If the imaginary line between the position of interest and its projection into the segment is not orthogonal to the segment geometry, it is considered that the point of interest does not belong to the road segment, and therefore, the closest point does not exist.

In case the closest point exist, its **segment distance** is always computed from the origin of the reference system of the road segment, following the path defined by the road segment geometry (center of the road segment) as shown in the previous image.  Do not confuse this distance with the closest euclidean distance between the point of interest and the closest point belonging to the lane.

The closest lane to a given pose is the lane belonging to the road segment that has the smallest euclidean distance to the given pose. Similarly, the closest road node to a given pose is the road node belonging to the closest lane whose lane link has the smallest euclidean distance to the given pose. In both cases the lane/link geometry is used to compute them. See the [COpendriveLane](opendrive_lane.md) and [COpendrveLink](opendrive_link.md) documentations for more details. In the previous image, the closest node and the closest lane are shown in blue.




