#ifndef _G2_SPLINE_H
#define _G2_SPLINE_H

#include "gradient.h"

#define DEFAULT_RESOLUTION     0.1

typedef struct
{
  double x;
  double y;
  double heading;
  double curvature;
}TPoint;

class CG2Spline
{
  private:
    TPoint start;
    TPoint end;
    double x_coeff[6];
    double y_coeff[6];
    bool generated;
    double *x;
    double *y;
    double *length;
    unsigned int num_points;
    double resolution;
    // optimization attributes
    CGradient curvature_grad;
    bool curvature_computed;
    double curvature_value;
    CGradient curvature_der_grad;
    bool curvature_der_computed;
    double curvature_der_value;
    CGradient distance_grad;
    double dist_x;
    double dist_y;
  protected:
    void normalize_angle(double &angle);
    double find_parameter(double length);
    TPoint evaluate_parameter(double u);
    double get_max_sample_point(boost::function<double (double)> p_function);
    double get_min_sample_point(boost::function<double (double)> p_function);
    double curvature(double u);
    double curvature_pow2(double u);
    double curvature_pow2_der(double u);
    double curvature_der(double u);
    double curvature_der_pow2(double u);
    double curvature_der_pow2_der(double u);
    double distance(double u);
    double distance_pow2(double u);
    double distance_pow2_der(double u);
  public:
    CG2Spline();
    CG2Spline(const CG2Spline &obj);
    CG2Spline(TPoint &start,TPoint &end);
    double get_resolution(void);
    void set_start_point(const TPoint &point);
    void get_start_point(TPoint &point);
    void set_end_point(const TPoint &point);
    void get_end_point(TPoint &point);
    void generate(double resolution,unsigned int iterations=10);
    void generate(double resolution,double length,unsigned int iterations=10);
    TPoint evaluate(double length);
    void evaluate_all(std::vector<double> &x, std::vector<double> &y,std::vector<double> &curvature,std::vector<double> &heading);
    double get_length(void);
    double get_max_curvature(double max_error=0.001,unsigned int max_iter=10);
    double get_max_curvature_der(double max_error=0.001,unsigned int max_iter=10);
    double find_closest_point(double x,double y,TPoint &point,double max_error=0.001,unsigned int max_iter=10);
    double find_closest_point(double x,double y,double max_error=0.001,unsigned int max_iter=10);
    void get_part(CG2Spline *spline,double start_length, double end_length=-1.0);
    CG2Spline& operator=(const CG2Spline &obj);
    ~CG2Spline();
};

#endif
