#include "gradient.h"
#include <math.h>

CGradient::CGradient()
{ 
  this->max_coord_inc=DEFAULT_MAX_INC;	
  this->max_coord=std::numeric_limits<double>::max();
  this->min_coord=-std::numeric_limits<double>::max();
}

CGradient::CGradient(double max_coord_inc)
{
  this->set_max_coord_inc(max_coord_inc);
  this->max_coord=std::numeric_limits<double>::max();
  this->min_coord=-std::numeric_limits<double>::max();
}

void CGradient::set_max_coord_inc(double max_inc)
{
  this->max_coord_inc=max_inc;
}

double CGradient::get_max_coord_inc(void)
{
  return this->max_coord_inc;
}

void CGradient::set_coord_constraints(double max,double min)
{
  this->max_coord=max;
  this->min_coord=min;
}

void CGradient::get_coord_constraints(double &max,double &min)
{
  max=this->max_coord;
  min=this->min_coord;
}

void CGradient::set_function(boost::function<double (double)> p_function)
{
  this->p_function=p_function;
}

void CGradient::set_function_der(boost::function<double (double)> p_function)
{
  this->p_function_der=p_function;
}

bool CGradient::compute(double max_func_error,unsigned int max_iter,double start_point,bool &beyond_limits,bool max)
{
  unsigned int iteration=0;
  double gamma=1.0,func_value,func_der_value,prev_func_value,prev_func_der_value;
  double coord,prev_coord,coord_inc;

  beyond_limits=false;
  coord=start_point;
  func_value=this->p_function(coord);
  func_der_value=this->p_function_der(coord);
  do{
    coord_inc=gamma*func_der_value;  
    if(coord_inc>this->max_coord_inc)
      coord_inc=this->max_coord_inc;
    else if(coord_inc<-this->max_coord_inc)
      coord_inc=-this->max_coord_inc;
    prev_coord=coord;
    if(max)
      coord+=coord_inc;
    else
      coord-=coord_inc;
    if((coord-this->max_coord)>max_func_error)
    {
      coord=this->max_coord;
      this->coordinate=coord;
      this->value=this->p_function(coord);
      beyond_limits=true;
      break;
    }
    else if((coord-this->min_coord)<-max_func_error)
    {
      coord=this->min_coord;
      this->coordinate=coord;
      this->value=this->p_function(coord);
      beyond_limits=true;
      break;
    }
    prev_func_value=func_value;
    func_value=this->p_function(coord);
    prev_func_der_value=func_der_value;
    func_der_value=this->p_function_der(coord);
    gamma=fabs((coord-prev_coord)/(func_der_value-prev_func_der_value));
    iteration++;
  }while((fabs(func_value-prev_func_value)>max_func_error || fabs(coord-prev_coord)>0.001) && iteration<max_iter);
  this->coordinate=coord;
  this->value=func_value;
  if(iteration==max_iter)
    return false;
  else
    return true;
}

double CGradient::get_value(void)
{
  return this->value;
}

double CGradient::get_coordinate(void)
{
  return this->coordinate;
}

CGradient::~CGradient()
{


}
